@@ -1,29 +1,7 @@
-package Catmandu::Importer::OAI::DC;
-
-use Catmandu::Sane;
-use Moo;
-
-has metadataPrefix => (is => 'ro' , default => sub { "oai_dc" });
-
-sub parse {
-    my ($self,$dom) = @_;
-
-    return undef unless defined $dom;
-
-    my $rec = {};
-
-    for ($dom->findnodes("./*")) {
-        my $name  = $_->localName;
-        my $value = $_->textContent;
-        push(@{$rec->{$name}}, $value);
-    }
-
-    $rec;
-}
-
 package Catmandu::Importer::OAI;
 
 use Catmandu::Sane;
+use Catmandu::Importer::OAI::DC;
 use Moo;
 use HTTP::OAI;
 use Data::Dumper;
@@ -42,7 +20,7 @@ has listIdentifiers => (is => 'ro');
 
 sub _build_oai {
     my ($self) = @_;
-    HTTP::OAI::Harvester->new(baseURL => $_[0]->url, resume => 1);
+    HTTP::OAI::Harvester->new(baseURL => $self->url, resume => 0);
 }
 
 sub _map_record {
@@ -89,6 +67,7 @@ sub _args {
         from => $self->from ,
         until => $self->until ,
     );
+
     for( keys %args ) {
         delete $args{$_} if !defined($args{$_}) || !length($args{$_});
     } 
@@ -96,9 +75,8 @@ sub _args {
     return %args;
 }
 
-sub generator {
+sub dry_run {
     my ($self) = @_;
-    return $self->dry ?
     sub {
         state $called = 0;
         return if $called;
@@ -110,15 +88,49 @@ sub generator {
                 verb => ($self->listIdentifiers ? 'ListIdentifiers' : 'ListRecords')
             )
         };
-    } : sub {
-        state $res = ($self->listIdentifiers
-            ? $self->oai->ListIdentifiers( $self->_args )
-            : $self->oai->ListRecords( $self->_args ));
-        if ($res->is_error) {
-            warn $res->message;
-            return;
+    };
+}
+
+sub oai_run {
+    my ($self) = @_;
+    sub {
+        state $stack = [];
+        state $resumptionToken = undef;
+        state $done  = 0;
+
+        my $fill_stack = sub {
+            push @$stack , shift;
+        };
+
+        if (@$stack <= 1 && $done == 0) {
+            my %args = $self->_args;
+
+            # Use the resumptionToken if one found on the last run, or if it was
+            # undefined (last record)
+            if (defined $resumptionToken) {
+                my $verb = $args{verb};
+                %args = (verb => $verb , resumptionToken => $resumptionToken);
+            }
+
+            my $res = $self->listIdentifiers 
+                ? $self->oai->ListIdentifiers( %args , onRecord => $fill_stack )
+                : $self->oai->ListRecords( %args , onRecord => $fill_stack );
+
+            if ($res->is_error) {
+                $self->log->error($res->message);
+                return undef;
+            }
+
+            if (defined $res->resumptionToken) {
+                $resumptionToken = $res->resumptionToken->resumptionToken;
+            }
+           
+            unless (defined $resumptionToken && length $resumptionToken) {
+                $done = 1;
+            }
         }
-        if (my $rec = $res->next) {
+
+        if (my $rec = shift @$stack) {
             if ($rec->isa('HTTP::OAI::Record')) {
                 return $self->_map_record($rec);
             } else {
@@ -129,10 +141,16 @@ sub generator {
                 }
             }
         }
-        return;
+
+        return undef;
     };
 }
 
+sub generator {
+    my ($self) = @_;
+    return $self->dry ? $self->dry_run : $self->oai_run;
+}
+
 =head1 NAME
 
 Catmandu::Importer::OAI - Package that imports OAI-PMH feeds
